using Microsoft.Extensions.Configuration;
using Phares.Shared;
using System.Text.Json;

namespace View_by_Distance.Rename.Models.Binder;

public class Configuration
{

    public bool? ForceMetadataLastWriteTimeToCreationTime { get; set; }
    public string[]? IgnoreExtensions { get; set; }
    public string? PersonBirthdayFormat { get; set; }
    public bool? PropertiesChangedForMetadata { get; set; }

    public override string ToString()
    {
        string result = JsonSerializer.Serialize(this, new JsonSerializerOptions() { WriteIndented = true });
        return result;
    }

    private static Models.Configuration Get(Configuration? configuration, Property.Models.Configuration propertyConfiguration)
    {
        Models.Configuration result;
        if (configuration is null) throw new NullReferenceException(nameof(configuration));
        if (configuration.ForceMetadataLastWriteTimeToCreationTime is null) throw new NullReferenceException(nameof(configuration.ForceMetadataLastWriteTimeToCreationTime));
        if (configuration.IgnoreExtensions is null) throw new NullReferenceException(nameof(configuration.IgnoreExtensions));
        if (configuration.PersonBirthdayFormat is null) throw new NullReferenceException(nameof(configuration.PersonBirthdayFormat));
        if (configuration.PropertiesChangedForMetadata is null) throw new NullReferenceException(nameof(configuration.PropertiesChangedForMetadata));
        result = new(propertyConfiguration,
                     configuration.ForceMetadataLastWriteTimeToCreationTime.Value,
                     configuration.IgnoreExtensions,
                     configuration.PersonBirthdayFormat,
                     configuration.PropertiesChangedForMetadata.Value);
        return result;
    }

    public static Models.Configuration Get(IsEnvironment isEnvironment, IConfigurationRoot configurationRoot, Property.Models.Configuration propertyConfiguration)
    {
        Models.Configuration result;
        Configuration? configuration;
        if (isEnvironment is null)
            configuration = configurationRoot.Get<Configuration>();
        else
        {
            string environmentName = IsEnvironment.GetEnvironmentName(isEnvironment);
            string section = string.Concat(environmentName, ":", nameof(Configuration));
            IConfigurationSection configurationSection = configurationRoot.GetSection(section);
            configuration = configurationSection.Get<Configuration>();
        }
        result = Get(configuration, propertyConfiguration);
        return result;
    }

}